package main

import "core:runtime"
import "core:fmt"
import "core:os"
import stbtt "vendor:stb/truetype"
import ui "vendor:microui"

import "gfx"

gfx_cx: ^gfx.cx = nil
ui_cx: ui.Context

last_mouse_left_down := false;
last_mouse_right_down := false;

rasterized_font_height: i32 = 16

frame_func :: proc "c" (mouse_x: int, mouse_y: int, mouse_left_down: bool, mouse_right_down: bool) {
    context = runtime.default_context()
    frame_width := gfx.frame_width(gfx_cx)
    frame_height := gfx.frame_height(gfx_cx)

    ui.input_mouse_move(&ui_cx, i32(mouse_x), i32(mouse_y));
    if mouse_left_down && !last_mouse_left_down {
        ui.input_mouse_down(&ui_cx, i32(mouse_x), i32(mouse_y), .LEFT);
    } else if !mouse_left_down && last_mouse_left_down {
        ui.input_mouse_up(&ui_cx, i32(mouse_x), i32(mouse_y), .LEFT);
    }

    last_mouse_left_down = mouse_left_down;
    last_mouse_right_down = mouse_right_down;

    ui.begin(&ui_cx)
    {
        defer ui.end(&ui_cx)

        if ui.begin_window(&ui_cx, "canvas", ui.Rect{ 0, 0, i32(frame_width), i32(frame_height) }, opt = { .EXPANDED, .NO_CLOSE, .NO_RESIZE, .NO_INTERACT, .NO_TITLE }) {
            defer ui.end_window(&ui_cx)

            ui.layout_row(&ui_cx, []i32 { 180, -1 }, -1)
            ui.begin_panel(&ui_cx, "top panel", opt = { .AUTO_SIZE })
            {
                defer ui.end_panel(&ui_cx)

                ui.layout_row(&ui_cx, []i32 { -1 }, 24);

                if ui.button(&ui_cx, "Mock Channel 1") >= { .SUBMIT } {
                    fmt.println("mock channel 1")
                }
                if ui.button(&ui_cx, "Mock Channel 2") >= { .SUBMIT } {
                    fmt.println("mock channel 2")
                }
                if ui.button(&ui_cx, "Mock Channel 3") >= { .SUBMIT } {
                    fmt.println("mock channel 3")
                }
            }

            ui.layout_begin_column(&ui_cx);
            {
                defer ui.layout_end_column(&ui_cx);

                ui.layout_row(&ui_cx, []i32 { -1 }, -64);

                ui.begin_panel(&ui_cx, "channel view")
                {
                    defer ui.end_panel(&ui_cx)
                    ui.text(&ui_cx, "TODO: put slack channel messages here")
                }

                ui.layout_row(&ui_cx, []i32 { -1 }, -1);
                ui.begin_panel(&ui_cx, "text edit view")
                {
                    defer ui.end_panel(&ui_cx)
                    ui.text(&ui_cx, "TODO: add text box here")
                }
            }
        }

        canvas := ui.get_container(&ui_cx, "canvas");
        if canvas != nil {
            canvas.rect.w = i32(frame_width);
            canvas.rect.h = i32(frame_height);
        }
    }

    cmd: ^ui.Command = nil
    for {
        variant, has_next := ui.next_command_iterator(&ui_cx, &cmd)
        if !has_next {
            break
        }

        switch v in variant {
            case ^ui.Command_Text: {
                gfx.queue_text(gfx_cx, v.str, { f32(v.pos.x), f32(v.pos.y-rasterized_font_height) }, 1000, 1000, { f32(v.color.r)/255.0, f32(v.color.g)/255.0, f32(v.color.r)/255.0, f32(v.color.a)/255.0 })
            }
            case ^ui.Command_Rect: {
                gfx.queue_ui_rect(gfx_cx, { f32(v.rect.x), f32(v.rect.y) }, { f32(v.rect.w), f32(v.rect.h) }, 0, { f32(v.color.r)/255.0, f32(v.color.g)/255.0, f32(v.color.r)/255.0, f32(v.color.a)/255.0 })
            }
            case ^ui.Command_Icon: {}
            case ^ui.Command_Clip: {}
            case ^ui.Command_Jump: {}
        }
    }
}

main :: proc() {
    gfx_cx = gfx.init_context(frame_func, 800, 600)

    font_data, success := os.read_entire_file_from_filename("./bin/JetBrainsMono-Medium.ttf")
    if !success {
        fmt.eprintln("failed to read font file")
        os.exit(1)
    }

    font: stbtt.fontinfo
    if stbtt.InitFont(&font, raw_data(font_data), 0) == false {
        fmt.eprintln("failed to init font")
        os.exit(1)
    }

    bitmap_size: i32 = 512
    bitmap := make([]u8, bitmap_size * bitmap_size)

    ascent, descent, line_gap: i32
    scale := stbtt.ScaleForPixelHeight(&font, f32(rasterized_font_height * 2))
    stbtt.GetFontVMetrics(&font, &ascent, &descent, &line_gap)

    gfx.add_glyph(gfx_cx, gfx.GpuGlyph {
        size = {f32(rasterized_font_height/4), 1},
        y_offset = f32(-rasterized_font_height),
    })

    x := rasterized_font_height / 4
    y: i32 = 0
    for i in 33..<33+96 {
        width, height, xoff, yoff: i32
        glyph_bitmap := stbtt.GetCodepointBitmap(
            &font, scale, scale, rune(i), &width, &height, &xoff, &yoff)

        if (x + width) >= bitmap_size {
            x = 0
            y += i32(f32(ascent - descent + line_gap) * scale)
        }

        xxx: i32 = x
        for xx in 0..<width {
            yyy: i32 = y
            for yy in 0..<height {
                bitmap[xxx + yyy * bitmap_size] =
                    glyph_bitmap[xx + yy * width]

                yyy += 1
            }

            xxx += 1
        }

        gfx.add_glyph(gfx_cx,
            gfx.GpuGlyph {
                atlas_position = {f32(x), f32(y)},
                size = {f32(width), f32(height)},
                position = {f32(x), f32(y)},
                y_offset = f32(yoff),
            }
        )

        x += width
    }
    gfx.push_texture_buffer(gfx_cx, u32(bitmap_size), u32(bitmap_size), raw_data(bitmap), u32(bitmap_size * bitmap_size))

    ui.init(&ui_cx)
    ui_cx.text_width = proc(font: ui.Font, str: string) -> i32 {
        return (rasterized_font_height/2) * i32(len(str)) + (rasterized_font_height/4) * 4
    }
    ui_cx.text_height = proc(font: ui.Font) -> i32 {
        return rasterized_font_height
    }

    // ui_cx.style = &ui.Style {
    //     size = ui.Vec2 { 0, 0 },
    //     padding = 5,
    //     spacing = 4,
    //     indent = 24,
    //     title_height = rasterized_font_height,
    //     scrollbar_size = 12,
    //     thumb_size = 8,
    //     colors = [ui.Color_Type]ui.Color {
    //         .TEXT = ui.Color         { 230, 230, 230, 255 },
    //         .BORDER = ui.Color       { 25,  25,  25,  255 },
    //         .WINDOW_BG = ui.Color    { 50,  50,  50,  255 },
    //         .TITLE_BG = ui.Color     { 25,  25,  25,  255 },
    //         .TITLE_TEXT = ui.Color   { 240, 240, 240, 255 },
    //         .PANEL_BG = ui.Color     { 0,   0,   0,   0   },
    //         .BUTTON = ui.Color       { 75,  75,  75,  255 },
    //         .BUTTON_HOVER = ui.Color { 95,  95,  95,  255 },
    //         .BUTTON_FOCUS = ui.Color { 115, 115, 115, 255 },
    //         .BASE = ui.Color         { 30,  30,  30,  255 },
    //         .BASE_HOVER = ui.Color   { 35,  35,  35,  255 },
    //         .BASE_FOCUS = ui.Color   { 40,  40,  40,  255 },
    //         .SCROLL_BASE = ui.Color  { 43,  43,  43,  255 },
    //         .SCROLL_THUMB = ui.Color { 30,  30,  30,  255 },
    //     },
    // };

    frame_func(0,0,false,false);
    frame_func(0,0,false,false);

    for(gfx.keep_running) {
        gfx.run_events(gfx_cx)
    }
}
